import { OrderedMap } from "immutable"

export enum InfoMessageType {
    BIG_INFO,
    BUILD_ERROR,
    BUILD_SUCCESSFUL
}

export type InfoMessage = {
    readonly id: string
    readonly type: InfoMessageType
    readonly message: string
};

export type InfoMessageMap = OrderedMap<string, InfoMessage>

export function newInfoMessageMap() {
    return OrderedMap<string, InfoMessage>()
}

/**
 * Delete all BIG_INFO messages from the InfoMessageMap
 * @param old the old map from where we would like to delete the items
 */
export function deleteBigInfoMessages(old: InfoMessageMap) {
    return old.withMutations(map => {
        old.filter(msg => msg.type === InfoMessageType.BIG_INFO)
            .forEach(msg => map.remove(msg.id))
    });

}


/**
 * Error message that is displayed to indicate that inheritance can't be done, because user has selected the same uml type twice 
 */
export const INFO_MESSAGE_ERROR_INHERITANCE_SAME_TYPE_SELECTED = "SameTypeSelectedForInheritanceId123579"

/**
 * Error message that is displayed if interface selection was wrong
 */
export const INFO_MESSAGE_ERROR_IMPLEMENTING_INTERFACE_SAME_TYPE_SELECTED = "SameTypeSelectedForInterfaceImplementation2378"

/**
 * Internally used to add / remove a InfoMessage asking the user to click on the canvas to add a new type
 */
export const INFO_MESSAGE_CLICK_ON_CANVAS_TO_ADD_TYPE = "SelectTypeForInheritance098767"

/**
 * Internally used to add / remove a InfoMessage asking the user to click on the canvas to add a new type
 */
export const INFO_MESSAGE_SELECT_TYPE_FOR_INHERITANCE = "SelectTypeForInheritance098767"

/**
 * Internally used to display a info message that asks the user to click on a class to start implementing an interface
 */
export const INFO_MESSAGE_FOR_INTERFACE_IMPLEMENTATION = "SelectClassForInterfaceImplementation897z7"


/**
 * Internally used to add / remove a InfoMessage asking the user to click on the canvas to add a new uml relation
 */
export const INFO_MESSAGE_SELECT_TYPE_FOR_RELATION = "SelectTypeForRelation"