import { List } from "immutable"

export enum MenuItemType {
    CLASS,
    INTERFACE,
    INHERITANCE,
    IMPLEMENT_INTERFACE,
    ASSOCIATION,
    COMPOSITION,
    AGGREGATION,
    SAVE_AND_RUN
}

/**
 * A Menu item type
 */
export type MenuItem = {
    readonly type: MenuItemType
    readonly name: string
    readonly selected: boolean
}

/**
 * Get the initial list of menut items in unselected state
 */
export function unselectedItems(): List<MenuItem> {
    return List(
        [{
            type: MenuItemType.CLASS,
            name: "Class",
            selected: false
        },

        {
            type: MenuItemType.INTERFACE,
            name: "Interface",
            selected: false
        },

        {
            type: MenuItemType.INHERITANCE,
            name: "Inheritance",
            selected: false
        },

        {
            type: MenuItemType.IMPLEMENT_INTERFACE,
            name: "Implement Interface",
            selected: false
        },

        {
            type: MenuItemType.ASSOCIATION,
            name: "Association",
            selected: false
        },

        {
            type: MenuItemType.COMPOSITION,
            name: "Composition",
            selected: false
        },

        {
            type: MenuItemType.AGGREGATION,
            name: "Aggregation",
            selected: false
        },
        {
            type: MenuItemType.SAVE_AND_RUN,
            name: "Save",
            selected: false
        }
        ]);
}

/**
 * Selects a menu item (unselects any previously selected one)
 * @param menuItemType The menuItem that should be selected
 */
export function selectMenuItem(menuItemType: MenuItemType): List<MenuItem> {
    const items = unselectedItems();
    const [index, menuItem] = items.findEntry(item => item.type === menuItemType);

    return items.set(index, {
        type: menuItem.type,
        name: menuItem.name,
        selected: true
    });
}


