import * as Rx from "@reactivex/rxjs";
import { Intent } from "./intent/Intent"
import { ViewState, initialViewState } from "./ViewState"
import { ViewStateReducer } from "./reducer/ViewStateReducer"
import { EditorUmlDocument, emptyUmlDocument } from "./model/EditorUmlDocument"
import {
    addUmlTypeAtPositionReducer, renameUmlTypeReducer, cancelRenamingUmlTypeReducer,
    startRenamingUmlTypeReducer, addUmlMethodReducer, startEditingUmlMethodReducer, cancelEditingUmlMethodReducer,
    changeUmlMethodSpecificationReducer, addUmlPropertyReducer, startEditingUmlPropertyReducer, cancelEditingUmlPropertyReducer,
    changeUmlPropertyReducer
} from "./reducer/UmlTypeReducer"
import {
    menuAddUmlTypeClickedReducer, menuInheritanceClickedReducer, menuAddUmlReactionClickedReducer,
    transactionSelectedFirstTypeForInheritanceReducer, transactionSelectedStartTypeForRelationReducer,
    menuImplementsInterfaceClickedReducer, transactionSelectedClassForInterfaceImplementationReducer
} from "./reducer/InteractionTransactionReducer"
import {
    deselectAllOnUmlCanvasClickReducer, selectUmlTypeOnUmCanvasReducer,
    selectUmlTypeAndMethodOnUmCanvasReducer, selectUmlTypeAndPropertyOnUmCanvasReducer,
    selectUmlInheritanceOnCanvasReducer, deleteSelectedElementsReducer,
    selectUmlImplementInterfacenReducer, selectUmlRelationReducer, moveSelectedElementsReducer,
    startEditinUmlRelationTextFieldReducer, cancelEditinUmlRelationTextFieldReducer, changeUmlRelationTextFieldReducer,
    cancelEverythingReducer
} from "./reducer/UmlCanvasReducer"
import { loadUmlDocumentReducer, saveUmlDocumentReducer } from "./reducer/LoadSaveReducer"
import { addInheritanceReducer } from "./reducer/UmlInheritanceReducer"
import { addInterfaceImplementationReducer } from "./reducer/UmlInterfaceImplementationReducer"
import {
    addRelationReducer, moveConnectionAnchorPointReducer,
    startMovingConnectionAnchorPointReducer, stopMovingConnectionAnchorPointReducer

} from "./reducer/UmlRelationReducer"
/**
 * This method is responsible to take intents and to transform them to a new state (by using reducers)
 * @param intents$ The stream of Intents to change the model (a.k.a ViewStae)
 */
export function modelFunction(intents$: Rx.Observable<Intent>, saveUmlDocument: (EditorUmlDocument) => void, loadUmlDocument: () => EditorUmlDocument): Rx.Observable<ViewState> {

    const logging = true;

    const initialState = initialViewState();

    if (logging) {
        intents$ = intents$.do(intent => console.log("Intent: " + intent)); // Add some console logging
    }

    const reducers$: Rx.Observable<ViewStateReducer> = Rx.Observable.merge(
        addUmlTypeAtPositionReducer(intents$, logging),
        renameUmlTypeReducer(intents$, logging),
        cancelRenamingUmlTypeReducer(intents$, logging),
        deselectAllOnUmlCanvasClickReducer(intents$, logging),
        cancelEverythingReducer(intents$, logging),
        menuAddUmlTypeClickedReducer(intents$, logging),
        menuInheritanceClickedReducer(intents$, logging),
        menuImplementsInterfaceClickedReducer(intents$, logging),
        menuAddUmlReactionClickedReducer(intents$, logging),
        transactionSelectedFirstTypeForInheritanceReducer(intents$, logging),
        transactionSelectedClassForInterfaceImplementationReducer(intents$, logging),
        transactionSelectedStartTypeForRelationReducer(intents$, logging),
        addInheritanceReducer(intents$, logging),
        selectUmlTypeOnUmCanvasReducer(intents$, logging),
        selectUmlTypeOnUmCanvasReducer(intents$, logging),
        selectUmlTypeAndMethodOnUmCanvasReducer(intents$, logging),
        selectUmlTypeAndPropertyOnUmCanvasReducer(intents$, logging),
        selectUmlInheritanceOnCanvasReducer(intents$, logging),
        selectUmlImplementInterfacenReducer(intents$, logging),
        selectUmlRelationReducer(intents$, logging),
        startRenamingUmlTypeReducer(intents$, logging),
        addUmlMethodReducer(intents$, logging),
        startEditingUmlMethodReducer(intents$, logging),
        cancelEditingUmlMethodReducer(intents$, logging),
        changeUmlMethodSpecificationReducer(intents$, logging),
        addUmlPropertyReducer(intents$, logging),
        startEditingUmlPropertyReducer(intents$, logging),
        cancelEditingUmlPropertyReducer(intents$, logging),
        changeUmlPropertyReducer(intents$, logging),
        deleteSelectedElementsReducer(intents$, logging),
        addInterfaceImplementationReducer(intents$, logging),
        addRelationReducer(intents$, logging),
        startEditinUmlRelationTextFieldReducer(intents$, logging),
        cancelEditinUmlRelationTextFieldReducer(intents$, logging),
        changeUmlRelationTextFieldReducer(intents$, logging),
        startMovingConnectionAnchorPointReducer(intents$, logging),
        stopMovingConnectionAnchorPointReducer(intents$, logging),
        moveConnectionAnchorPointReducer(intents$, logging),
        moveSelectedElementsReducer(intents$, logging),
        loadUmlDocumentReducer(intents$, logging, loadUmlDocument),
        saveUmlDocumentReducer(intents$, logging, saveUmlDocument)
    );

    return reducers$
        .scan((oldState, reducerFunc) => reducerFunc(oldState), initialState)
        .do(newState => {
            if (logging) console.log("New State " + JSON.stringify(newState, null, 1))
        });
}


