import { List, Map } from "immutable"
import { MenuItem, unselectedItems } from "./MenuItem"
import { EditorUmlDocument, emptyUmlDocument } from "./model/EditorUmlDocument"
import { InteractionTransaction, NoInteractionTransactionRunning } from "./interaction/InteractionTransaction"
import { ErrorMessage, ErrorsMap, newErrorsMap } from "./ErrorMessage"
import { InfoMessage, InfoMessageMap, newInfoMessageMap } from "./InfoMessage"
import { SelectedElementsMap, newSelectedElementsMap } from "./SelectedElement"

export interface ViewState {
    readonly menuItems: List<MenuItem>;
    readonly umlDocument: EditorUmlDocument;
    readonly currentInteractionTransaction: InteractionTransaction;
    readonly errors: ErrorsMap; // The key is the id of the element, the value is the error message
    readonly infoMessages: Map<string, InfoMessage>; // Info messages that in the app to guide the user through the app
    readonly selectedElements: SelectedElementsMap; // The map containing the elements that are selected
}

/**
 * The inital view state
 */
export function initialViewState(): ViewState {
    return {
        menuItems: unselectedItems(),
        umlDocument: emptyUmlDocument(),
        currentInteractionTransaction: NoInteractionTransactionRunning.getInstance(),
        errors: newErrorsMap(),
        infoMessages: newInfoMessageMap(),
        selectedElements: newSelectedElementsMap()
    }
}