import { Intent } from "./Intent"
import { UmlType } from "../model/EditorUmlClass"
import { MenuItemType } from "../MenuItem"


/**
 * An intent that indicates that we are going to add a UML Type
 */
export class StartAddingUmlTypeIntent implements Intent {
    readonly menuItemType: MenuItemType;
    readonly umlType: UmlType;

    constructor(menuItemType: MenuItemType, umlType: UmlType) {
        this.menuItemType = menuItemType;
        this.umlType = umlType;
    }

    toString(): string {
        return "StartAddingUmlTypeIntent of " + this.umlType;
    }
}

/**
 * Intent that inidicates that we are going to add a UML inheritance.
 * 
 * Successor is FirstElementSelectedIntent
 */
export class StartSelectingElementsIntent implements Intent {
    readonly menuItemType: MenuItemType;

    constructor(menuItemType: MenuItemType) {
        this.menuItemType = menuItemType;
    }

    toString(): string {
        return "StartSelectingElementsIntent for "+this.menuItemType;
    }
}

/**
 * Predecessor is StartSelectingElementsIntent.
 * Successor is SecondElementSelectedIntent.
 */
export class FirstElementSelectedIntent implements Intent {
    readonly menuItemType: MenuItemType;
    readonly firstSelectedElementId: string

    constructor(menuItemType: MenuItemType, firstSelectedElementId: string) {
        this.menuItemType = menuItemType;
        this.firstSelectedElementId = firstSelectedElementId;
    }

    toString(): string {
        return "FirstElementSelectedIntent for menu item " + this.menuItemType + " ; selected element: " + this.firstSelectedElementId;
    }
}

/**
 * predecessor is FirstElementSelectedIntent.
 */
export class SecondElementSelectedIntent implements Intent {
    readonly menuItemType: MenuItemType;
    readonly firstSelectedElementId: string
    readonly secondSelectedElementId: string

    constructor(menuItemType: MenuItemType, firstSelectedElementId: string, secondSelectedElementId: string) {
        this.menuItemType = menuItemType;
        this.firstSelectedElementId = firstSelectedElementId;
        this.secondSelectedElementId = secondSelectedElementId;
    }

    toString(): string {
        return "SecondElementSelectedIntent for menu item " + this.menuItemType + " ; selected elements: " + this.firstSelectedElementId + " , " + this.secondSelectedElementId;
    }
}