import { Intent } from "./Intent"
import { EditorUmlType } from "../model/EditorUmlClass"




/**
 * The intent to change the name of an uml type
 */
export class RenameUmlTypeIntent implements Intent {
    readonly umlType: EditorUmlType;
    readonly newName: string;

    constructor(umlType: EditorUmlType, newName: string) {
        this.umlType = umlType;
        this.newName = newName;
    }


    toString(): string {
        return "RenameUmlTypeIntent: old name = '" + this.umlType.name + "' - new name '" + this.newName + "'";
    }
}

/**
 * The intent to cancel renaming of an uml type
 */
export class CancelRenameUmlTypeIntent implements Intent {
    readonly umlType: EditorUmlType;

    constructor(umlType: EditorUmlType) {
        this.umlType = umlType;
    }


    toString(): string {
        return `CancelRenameUmlTypeIntent on ${this.umlType.name}`;
    }
}

/**
 *  The intent to start renaming a uml type
 */
export class StartRenamingUmlTypeIntent implements Intent {
    readonly umlType: EditorUmlType

    constructor(umlType: EditorUmlType) {
        this.umlType = umlType;
    }

    toString(): string {
        return "StartRenamingUmlTypeIntent for '" + this.umlType.name + "'";
    }
}

/**
 * The intent to add a uml method to a uml class
 */
export class AddUmlMethodIntent implements Intent {
    readonly umlType: EditorUmlType

    constructor(umlType: EditorUmlType) {
        this.umlType = umlType;
    }

    toString(): string {
        return "AddUmlMethodIntent. Add method to " + this.umlType.name;
    }
}


/**
 * The intent to edit a uml method
 */
export class StartEditingUmlMethodIntent implements Intent {
    readonly umlType: EditorUmlType
    readonly methodId: string

    constructor(umlType: EditorUmlType, methodId: string) {
        this.umlType = umlType;
        this.methodId = methodId;
    }

    toString(): string {
        return "StartEditingUmlMethodIntent for type " + this.umlType.name + " and method with id = " + this.methodId;
    }
}


/**
 * The intent to cancel editing a uml method
 */
export class CancelEditingUmlMethodIntent implements Intent {
    readonly umlType: EditorUmlType
    readonly methodId: string

    constructor(umlType: EditorUmlType, methodId: string) {
        this.umlType = umlType;
        this.methodId = methodId;
    }

    toString(): string {
        return "CancelEditingUmlMethodIntent for type " + this.umlType.name + " and method with id = " + this.methodId;
    }
}


/**
 * The intent to change the method specification of a uml method
 */
export class ChangeUmlMethodIntent implements Intent {
    readonly umlType: EditorUmlType
    readonly methodId: string
    readonly newMethodSpec: string

    constructor(umlType: EditorUmlType, methodId: string, newMethodSpec: string) {
        this.umlType = umlType;
        this.methodId = methodId;
        this.newMethodSpec = newMethodSpec;
    }

    toString(): string {
        return "ChangeUmlMethodIntent for type " + this.umlType.name + " to change method to " + this.newMethodSpec;
    }
}

/**
 * The intent to change the method specification of a uml method
 */
export class AddUmlPropertyIntent implements Intent {
    readonly umlType: EditorUmlType

    constructor(umlType: EditorUmlType) {
        this.umlType = umlType;
    }

    toString(): string {
        return "AddUmlPropertyIntent for type " + this.umlType.name;
    }
}

/**
 * The intent to change the property specification of a uml property
 */
export class ChangeUmlPropertyIntent implements Intent {
    readonly umlType: EditorUmlType
    readonly propertyId: string
    readonly newPropertySpec: string

    constructor(umlType: EditorUmlType, methodId: string, newPropertySpec: string) {
        this.umlType = umlType;
        this.propertyId = methodId;
        this.newPropertySpec = newPropertySpec;
    }

    toString(): string {
        return "ChangeUmlPropertyIntent for type " + this.umlType.name + " to change method to " + this.newPropertySpec;
    }
}


/**
 * The intent to edit a uml property
 */
export class StartEditingUmlPropertyIntent implements Intent {
    readonly umlType: EditorUmlType
    readonly propertyId: string

    constructor(umlType: EditorUmlType, propertyId: string) {
        this.umlType = umlType;
        this.propertyId = propertyId;
    }

    toString(): string {
        return "StartEditingUmlPropertyIntent for type " + this.umlType.name + " and method with id = " + this.propertyId;
    }
}


/**
 * The intent to cancel editing a uml property
 */
export class CancelEditingUmlPropertyIntent implements Intent {
    readonly umlType: EditorUmlType
    readonly propertyId: string

    constructor(umlType: EditorUmlType, propertyId: string) {
        this.umlType = umlType;
        this.propertyId = propertyId;
    }

    toString(): string {
        return "CancelEditingUmlPropertyIntent for type " + this.umlType.name + " and method with id = " + this.propertyId;
    }
}
