import { OrderedMap } from "immutable"
import { EditorUmlTypeMethod } from "./EditorUmlTypeMethod";
import { EditorUmlClassProperty } from "./EditorUmlClassProperty";


/**
 * An internal enum to distinguish between UML Class and UML Interface
 */
export enum UmlType {
    CLASS,
    INTERFACE
}

/**
 * Converts a UmlType into a human readable string
 * @param type the UmlType
 *
 */
export function umlTypeToHumanString(type : UmlType) : string {
    switch(type){
        case UmlType.CLASS: return "Class"
        case UmlType.INTERFACE: return "Interface"
        default: throw Error("Unknown umltype "+type)
    }
}

/**
 * This class represents a UML Class.
 * 
 */
export type EditorUmlType = {

    readonly id: string
    readonly x: number;
    readonly y: number;
    readonly name: string;
    readonly type: UmlType,
    readonly methods: OrderedMap<string, EditorUmlTypeMethod>
    readonly properties: OrderedMap<string, EditorUmlClassProperty>
};
