import { UmlType, EditorUmlType } from "./EditorUmlClass"
import { AnchorPoint } from "./AnchorPoint"
import { OrderedSet } from "immutable"
import { Point } from "./Point"

export enum UmlRelationType {
    AGGREGATION,
    COMPOSITION,
    ASSOCIATION
}

export function relationTypeToString(type: UmlRelationType) {
    switch (type) {
        case UmlRelationType.AGGREGATION: return "AGGREGATION"
        case UmlRelationType.COMPOSITION: return "COMPOSITION"
        case UmlRelationType.ASSOCIATION: return "ASSOCIATION"
        default: throw new Error("Unknown UmlRelationType " + type)
    }
}

export function relationTypeFromString(type: string): UmlRelationType {
    if (type === "ASSOCIATION")
        return UmlRelationType.ASSOCIATION
    else if (type === "COMPOSITION")
        return UmlRelationType.COMPOSITION
    else if (type === "AGGREGATION")
        return UmlRelationType.AGGREGATION

    throw new Error("Unknown UmlRelationType "+type);
}

export type EditorUmlRelation = {
    readonly id: string,
    readonly relationType: UmlRelationType,
    readonly startTypeId: string,
    readonly endTypeId: string,
    readonly startTypeAnchorPoint: AnchorPoint,
    readonly endTypeAnchorPoint: AnchorPoint,
    readonly startMultiplicity: string,
    readonly endMultiplicity: string,
    readonly points: OrderedSet<Point>
}

