import { ViewStateReducer, logReducer } from "../reducer/ViewStateReducer"
import * as Rx from "@reactivex/rxjs";
import { Intent } from "../intent/Intent"
import { SaveUmlDocumentIntent, LoadUmlDocumentIntent } from "../intent/LoadSaveIntents"
import { ViewState } from "../ViewState"
import { unselectedItems } from '../MenuItem'
import { umlDocumentFromJson } from '../model/Deserializer'
import { umlDocumentToJson } from '../model/Serializer'
import { NoInteractionTransactionRunning } from '../interaction/InteractionTransaction'
import { EditorUmlDocument } from '../model/EditorUmlDocument'

/**
 * Opens the uml document file.
 * Emits the read document as new state.
 * @param intent$ The intent observable stream
 */
export function loadUmlDocumentReducer(intent$: Rx.Observable<Intent>, log: boolean, loadFnc: () => EditorUmlDocument): Rx.Observable<ViewStateReducer> {
    return intent$
        .filter(intent => intent instanceof LoadUmlDocumentIntent)
        .map(intent => intent as LoadUmlDocumentIntent)
        .do(i => logReducer(log, "loadUmlDocumentReducer"))
        .switchMap(intent => Rx.Observable.of((oldState: ViewState): ViewState => {

            // TODO maybe async

            return {
                menuItems: unselectedItems(),
                umlDocument: loadFnc(),
                currentInteractionTransaction: NoInteractionTransactionRunning.getInstance(),
                errors: oldState.errors,
                selectedElements: oldState.selectedElements,
                infoMessages: oldState.infoMessages
            };
        }));
}


/**
 * Opens the uml document file.
 * Emits the read document as new state.
 * @param intent$ The intent observable stream
 */
export function saveUmlDocumentReducer(intent$: Rx.Observable<Intent>, log: boolean, saveFnc: (EditorUmlDocument) => void): Rx.Observable<ViewStateReducer> {
    return intent$
        .filter(intent => intent instanceof SaveUmlDocumentIntent)
        .map(intent => intent as SaveUmlDocumentIntent)
        .do(i => logReducer(log, "saveUmlDocumentReducer"))
        .switchMap(intent => Rx.Observable.of((oldState: ViewState): ViewState => {
            // TODO maybe async observable / promise
            saveFnc(oldState.umlDocument)
            return oldState
        }));
}
